-------------------------------------------------------------------------------
-- Filename:     slot.vhd
-- History:
--   2006-7-14     First Version
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity slot is
  port (
    SYS_CLK                 : in  std_logic;
    SYS_RST                 : in  std_logic;

    CODE                    : in  std_logic_vector(0 to 3);
    nSW                     : in  std_logic_vector(1 to 3);
    lear_chat_timing        : in  std_logic; -- count_out(9)
    slot_turn_slow_timing   : in  std_logic; -- count_out(0)
    slot_turn_middle_timing : in  std_logic; -- count_out(1)
    slot_turn_high_timing   : in  std_logic; -- count_out(2)

    seg_in1                 : out std_logic_vector(0 to 3);
    seg_in2                 : out std_logic_vector(0 to 3);
    seg_in3                 : out std_logic_vector(0 to 3);
    le_trig                 : out std_logic
  );

end entity slot;

architecture imp of slot is

signal lear_chat_timing_reg : std_logic;
type sw_reg_type is array(0 to 2,1 to 3) of std_logic;
signal sw_reg               : sw_reg_type;
signal sw_hi                : std_logic_vector(1 to 3);
signal sw_lo                : std_logic_vector(1 to 3);
signal detect_redey         : std_logic;
signal detect_turn          : std_logic;
signal detect_stop          : std_logic;
type stat_type is (SLOT_STOP, SLOT_READY, SLOT_TURN);
signal stat                 : stat_type;
signal sw_stop              : std_logic_vector(1 to 3);
signal slot_turn_timing     : std_logic;
signal slot_turn_timing_reg : std_logic;
subtype SEG7_sub_type is std_logic_vector(0 to 3);
type seg7_3_type is array(1 to 3) of SEG7_sub_type;
signal seg7_val_w           : seg7_3_type;

begin

  process (SYS_CLK) is
  begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_RST = '1' then
      lear_chat_timing_reg <= '0';
    else
      lear_chat_timing_reg <= lear_chat_timing;
    end if;
  end if;
  end process;

FORGEN0 : for I in 1 to 3 generate
  process (SYS_CLK) is
  begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_RST = '1' then
      sw_reg(0,I) <= '0';
      sw_reg(1,I) <= '0';
      sw_reg(2,I) <= '0';
    else
      if (lear_chat_timing = '1') and (lear_chat_timing_reg = '0') then
        sw_reg(0,I) <= nSW(I);
        sw_reg(1,I) <= sw_reg(0,I);
        sw_reg(2,I) <= sw_reg(1,I);
      end if;
    end if;
  end if;
  end process;
  sw_hi(I) <= sw_reg(0,I) and sw_reg(1,I) and sw_reg(2,I);
  sw_lo(I) <= not (not sw_reg(0,I) and not sw_reg(1,I) and not sw_reg(2,I));
end generate;

  detect_redey <= (sw_hi(1) and sw_hi(2))or(sw_hi(1) and sw_hi(3))or(sw_hi(2) and sw_hi(3));
  detect_turn <= not sw_lo(1) and not sw_lo(2) and not sw_lo(3);
  detect_stop <= sw_stop(1) and sw_stop(2) and sw_stop(3);

  process (SYS_CLK) is
  begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_RST = '1' then
        stat <= SLOT_STOP;
    else
      case stat is
      when SLOT_STOP =>
        if detect_redey = '1' then
          stat <= SLOT_READY;
        end if;
      when SLOT_READY =>
        if detect_turn = '1' then
          stat <= SLOT_TURN;
        end if;
      when SLOT_TURN =>
        if detect_stop = '1' then
          stat <= SLOT_STOP;
        end if;
      when others => 
        stat <= SLOT_STOP;
      end case;
    end if;
  end if;
  end process;

FORGEN1 : for I in 1 to 3 generate
  process (SYS_CLK) is
  begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_RST = '1' then
      sw_stop(I) <= '0';
    else
      if (stat = SLOT_TURN) and (sw_hi(I) = '1') then
        sw_stop(I) <= '1';
      elsif stat = SLOT_READY then
        sw_stop(I) <= '0';
      end if;
    end if;
  end if;
  end process;
end generate;

  process (CODE, slot_turn_slow_timing, slot_turn_middle_timing, slot_turn_high_timing) is
  begin
  case CODE is
  when x"0" =>
    slot_turn_timing <= slot_turn_slow_timing;
  when x"1" =>
    slot_turn_timing <= slot_turn_middle_timing;
  when others => 
    slot_turn_timing <= slot_turn_high_timing;
  end case;
  end process;

  process (SYS_CLK) is
  begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_RST = '1' then
      slot_turn_timing_reg <= '0';
    else
      slot_turn_timing_reg <= slot_turn_timing;
    end if;
  end if;
  end process;

FORGEN2 : for I in 1 to 3 generate
  process (SYS_CLK) is
  begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_RST = '1' then
      seg7_val_w(I) <= x"0";
    else
      if (slot_turn_timing = '1') and (slot_turn_timing_reg = '0') then
        if (stat = SLOT_TURN) and (sw_stop(I) = '0') then
          seg7_val_w(I) <= seg7_val_w(I) + 1;
        end if;
      end if;
    end if;
  end if;
  end process;
end generate;

  seg_in1 <= seg7_val_w(3);
  seg_in2 <= seg7_val_w(2);
  seg_in3 <= seg7_val_w(1);

  le_trig <= '1' when (seg7_val_w(1) = seg7_val_w(2)) and (seg7_val_w(1) = seg7_val_w(3)) else '0';

end architecture imp;
